﻿// WiiMote Receiver Class
//
// Version 2.0
// Author: Mario Klingemann <mario@quasimondo.com>
// Maintained By: Adam Saltsman <adam@lastchancemedia.com>
//
// This class allows for the use of the WiiMote in Flash applications,
// specifically games.  Must be used in conjunction with WiiBounce.swf.
//
// Please see MarioToo_Demo.as for usage examples!

import mx.utils.Delegate;
import mx.events.EventDispatcher;

class com.LCM.mariotoo.FlashMX.WiiMote
{
	private var lc:LocalConnection;

	public var BUTTON_A:Number = 13;
	public var BUTTON_MINUS:Number = 170;
	public var BUTTON_B:Number = 171;
	public var BUTTON_1:Number = 172;
	public var BUTTON_2:Number = 173;
	public var BUTTON_PLUS:Number = 174;
	public var BUTTON_UP:Number = 175;
	public var BUTTON_DOWN:Number = 176;
	public var BUTTON_RIGHT:Number = 177;
	public var BUTTON_LEFT:Number = 178;
	public var isAWiiMote : Boolean = false;
	
	static public var addEventListener:Function;
	static public var removeEventListener:Function;
	static private var dispatchEvent:Function;
	
	private var keyStats:Array;
	private var binds:Array;
	
	private var classic = false;
	private var usingKeyboard = false;
	private var isPrimary = false;	// True if this is the wiimote in charge of the mouse clicks, used to filter mouse presses
									// otherwise when button A is pressed on pad one all 4 will generate button presses
	private var _forcePrimary = false; //required for pc usage as normal.Is automatically set to WiiMote ID 1
	
	function WiiMote(ID:Number)
	{
		if(ID == 1)
			_forcePrimary = true;

		EventDispatcher.initialize( this );
		lc = new LocalConnection();
		lc.connect("wii"+ID);
		lc.setState = Delegate.create( this, setState );
		Mouse.addListener( this );
		Key.addListener( this );
		keyStats = [];
		binds = [];
	}
	
	public function isDown( keyCode ):Boolean
	{
		var tmp = keyCode;
		if(!usingKeyboard && classic)
		{
			switch(tmp)
			{
				case BUTTON_UP:
					tmp = BUTTON_RIGHT;
					break;
				case BUTTON_RIGHT:
					tmp = BUTTON_DOWN;
					break;
				case BUTTON_DOWN:
					tmp = BUTTON_LEFT;
					break;
				case BUTTON_LEFT:
					tmp = BUTTON_UP;
					break;
			}
		}
		return ( keyStats[tmp] == true );
	}
	
	private function setState(buttons,dpad):Void
	{
		keyStats[BUTTON_PLUS] = (buttons >= 64);
		if(keyStats[BUTTON_PLUS])
			buttons -= 64;
		keyStats[BUTTON_2] = (buttons >= 32);
		if(keyStats[BUTTON_2])
			buttons -= 32;
		keyStats[BUTTON_1] = (buttons >= 16);
		if(keyStats[BUTTON_1])
			buttons -= 16;
		keyStats[BUTTON_B] = (buttons >= 8);
		if(keyStats[BUTTON_B])
			buttons -= 8;
		keyStats[BUTTON_MINUS] = (buttons >= 4);
		if(keyStats[BUTTON_MINUS])
			buttons -= 4;
		keyStats[BUTTON_A] = (buttons >= 2);
		
		isAWiiMote = (dpad >= 64);				/// a little hacky, but the end user needs to know if a wiimote is connected,
									// again if player one runs out of battery or disconnects this call will automatically update the status
		if(isAWiiMote)
			dpad -= 64;
		isPrimary = false;
		isPrimary = (dpad >= 32);				/// again a little hacky, but the primary wii mote can change: if player one runs out of battery or disconnects his/her
		if(isPrimary)						// wiimote player 2 will be primary, therfore we have to update via javascript to see the primary wiimote
			dpad -= 32;
		
		keyStats[BUTTON_LEFT] = (dpad >= 16);
		if(keyStats[BUTTON_LEFT])
			dpad -= 16;
		keyStats[BUTTON_RIGHT] = (dpad >= 8);
		if(keyStats[BUTTON_RIGHT])
			dpad -= 8;
		keyStats[BUTTON_DOWN] = (dpad >= 4);
		if(keyStats[BUTTON_DOWN])
			dpad -= 4;
		keyStats[BUTTON_UP] = (dpad >= 2);
	}

	private function onMouseDown():Void
	{
		if (isPrimary || _forcePrimary)
			keyStats[BUTTON_A] = true;
	}
	
	private function onMouseUp():Void
	{
		if (isPrimary || _forcePrimary)
			keyStats[BUTTON_A] = false;
	}
	
	private function onKeyDown():Void
	{
		keyStats[binds[Key.getCode()]] = true;
		usingKeyboard = true;
	}
	
	private function onKeyUp():Void
	{
		keyStats[binds[Key.getCode()]] = false;
		usingKeyboard = true;
	}
	
	public function setClassic(c):Void
	{
		classic = c;
	}
	
	public function bind(b,k):Void
	{
		binds[k] = b;
	}
	
	public function unbind(k):Void
	{
		binds[k] = null;
	}
}